@@ -244,17 +244,20 @@ public final class Native extends NativeBase {
    * Initialize a record batch reader for a PartitionedFile
    *
    * @param filePath
-   * @param start
-   * @param length
+   * @param starts
+   * @param lengths
    * @return a handle to the record batch reader, used in subsequent calls.
    */
   public static native long initRecordBatchReader(
       String filePath,
       long fileSize,
-      long start,
-      long length,
+      long[] starts,
+      long[] lengths,
+      byte[] filter,
       byte[] requiredSchema,
-      String sessionTimezone);
+      byte[] dataSchema,
+      String sessionTimezone,
+      int batchSize);
 
   // arrow native version of read batch
   /**
